#
#	Copyright (C) 2007. PARP Research Group.
#	<http://perception.inf.um.es>
#	University of Murcia, Spain.
#
#	This file is part of the QVision library.
#
#	QVision is free software: you can redistribute it and/or modify
#	it under the terms of the GNU Lesser General Public License as
#	published by the Free Software Foundation, version 3 of the License.
#
#	QVision is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU Lesser General Public License for more details.
#
#	You should have received a copy of the GNU Lesser General Public
#	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
#

################################
#
#	File qvision.pri
#

################################
## GET CONFIGURATION
include(config.pri)

################################
## Compilation configuration

message(--------------------------)
# Library include and binary file paths.
message(Install path set to: $$INSTALL_PATH)
DEPENDPATH += $$INSTALL_PATH/src/
INCLUDEPATH += $$INSTALL_PATH/include/

# Compilation options.
!release	{
		message(Compilation mode: DEBUG)
		# For gprof debugging
		QMAKE_LFLAGS_DEBUG += -p -pg
		QMAKE_CXXFLAGS_DEBUG += -p -pg
	
		# active warning messages when compiling
		CONFIG += warn_on
		}
else		{
		message(Compilation mode: RELEASE)
		}

!qt_debug_printing	{
			message(Debug printing: disabled)
			DEFINES += QT_NO_DEBUG_OUTPUT
			}
else			{
			message(Debug printing: enabled)
			}

message(Performance optimization level: $$PERFORMANCE_OPTIMIZATION)
QMAKE_CXXFLAGS_RELEASE -= -O2
QMAKE_CXXFLAGS_RELEASE += -$$PERFORMANCE_OPTIMIZATION

# Opengl
QT += opengl

# QWT
QWT_LIB_DIR = $$dirname(QWT_LIB_BINARY)
TEMP1 = $$basename(QWT_LIB_BINARY)
TEMP2 = $$replace(TEMP1, .so, )
QWT_LIB_NAME = $$replace(TEMP2, lib, )

!exists($$QWT_INCLUDE): \
	error(Specified QWT include directory ($$QWT_INCLUDE) does not exists. Set a correct path at the file 'config.pri'.)

!exists($$QWT_LIB_BINARY): \
	error(Specified QWL library binary ($$QWT_LIB_BINARY) does not exists. Set a correct path at the file 'config.pri'.)

message(Path to QWT include directory: $$QWT_INCLUDE)
message(Path to QWT library binary: $$QWT_LIB_BINARY)

INCLUDEPATH	+= $$QWT_INCLUDE
LIBS		+= -L$$QWT_LIB_DIR -l$$QWT_LIB_NAME
!icc:LIBS	+= -Wl,-rpath $$QWT_LIB_DIR
win32:DEFINES	+= QT_DLL QWT_DLL

# IPP
!exists($$IPP_PATH): \
	error(IPP library was not found at specified path: $$IPP_PATH . Set \
		a correct path in the IPP_PATH variable at the config.pri file)
!exists($$IPP_PATH/include): \
	error(Include files directory was not found at the specified path for the IPP library: $$IPP_PATH/include . \
		Perhaps it is not the installation directory for the IPP?)
!exists($$IPP_PATH/sharedlib): \
	error(Shared library binaries directory was not found at the specified path for the IPP library: \ 
		$$IPP_PATH/sharedlib . Perhaps it is not the installation directory for the IPP?)

message(Path to IPP: $$IPP_PATH)

INCLUDEPATH += $$IPP_PATH/include
unix	{
	LIBS		+= -L$$IPP_PATH/sharedlib
	!icc:LIBS	+= -Wl,-rpath $$IPP_PATH/sharedlib
	exists($$find(IPP_PATH, em64t)):	LIBS += -lguide -lippiem64t -lippcoreem64t -lippsem64t -lippccem64t -lippvcem64t -lippcvem64t
	exists($$find(IPP_PATH, ia32)):		LIBS += -lguide -lippi -lippcore -lipps -lippcc -lippvc -lippcv
	}

# GSL
LIBS += -lgsl -lgslcblas -lm

# Activate pkgconfig application usage
CONFIG += link_pkgconfig	# Enable pkg-config usage through the PKGCONFIG variable.

################################
## PACKAGE DEPENDANT DEFINES

qvmplayercamera	{
		message(--------------------------)
		message(MPlayer functionality enabled)
		
		!exists($$MPLAYER_BINARY_PATH)	{
						error(QVMPlayer binary was not found at: $$MPLAYER_BINARY_PATH . Set \
							a correct path in the MPLAYER_BINARY_PATH variable at the config.pri file)
						}

		message(QVMPlayer binary found at: $$MPLAYER_BINARY_PATH)
		# Preprocessor variables. Can be used with #ifdef preprocesor directives.
		DEFINES += MPLAYER_BINARY_PATH=\\\"$$MPLAYER_BINARY_PATH\\\"
		}

# Needed for Intel's CC
icc	{
	message(--------------------------)
	message(ICC compiler enabled)

	exists($$ICC_BINARY)	{
				message(icc binary found at: $$MPLAYER_BINARY_PATH)
				}
	else	{
		error(icc binary was not found at: $$ICC_BINARY \
			Set a correct path in the ICC_BINARY variable at the config.pri file)
		}

	QMAKE_CXX = $$ICC_BINARY
	QMAKE_LINK = $$ICC_BINARY	# Undocumented variable. That is nice, Qt trolls.
	}

qvcuda {
	message(--------------------------)
	message(CUDA functionality enabled)

	INCLUDEPATH += $$CUDA_DIR/include $$CUDA_UTIL_DIR/common/inc
	QMAKE_LIBDIR += $$CUDA_DIR/lib $$CUDA_UTIL_DIR/lib
	LIBS += -lcudart -lcutil -Wl,-rpath $$CUDA_DIR/lib

	cuda.output = ${OBJECTS_DIR}${QMAKE_FILE_BASE}_cuda.obj
	cuda.commands = $${CUDA_DIR}/bin/nvcc -c -Xcompiler		\
			$$join(QMAKE_CXXFLAGS,",")			\
			$$join(QMAKE_INCDIR,'" -I "','-I "','"')	\
			$$join(INCLUDEPATH,'" -I "','-I "','"')		\
			${QMAKE_FILE_NAME} -o ${QMAKE_FILE_OUT}

	cuda.dependcy_type = TYPE_C
	cuda.depend_command = nvcc -M -Xcompiler $$join(QMAKE_CXXFLAGS,",")	\
				$$join(INCLUDEPATH,'" -I "','-I "','"')		\
				${QMAKE_FILE_NAME} | sed "s,^.*: ,," | sed "s,^ *,," | tr -d '\\\n'

	cuda.input = CUDA_SOURCES
	QMAKE_EXTRA_UNIX_COMPILERS += cuda
	}

opencv	{
	message(--------------------------)
	message(OpenCV compatibility enabled)
	PKGCONFIG += opencv		# Include necessary directories and binary files for OpenCV.
	DEFINES += OPENCV
	}

cgal	{
	message(--------------------------)
	message(CGAL library compatibility enabled)
	LIBS += -lCGAL
	}

message(--------------------------)
