/*
 *	Copyright (C) 2007, 2008, 2009. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <QDebug>
#include <QVImage>

#include <iostream>
template <> const char * QVImage<uChar,1>::getTypeQString()	const	{ return "QVImage<uChar,1>"; }
template <> const char * QVImage<uChar,3>::getTypeQString()	const	{ return "QVImage<uChar,3>"; }
template <> const char * QVImage<uShort,1>::getTypeQString()	const	{ return "QVImage<uShort,1>"; }
template <> const char * QVImage<uShort,3>::getTypeQString()	const	{ return "QVImage<uShort,3>"; }
template <> const char * QVImage<sShort,1>::getTypeQString()	const	{ return "QVImage<sShort,1>"; }
template <> const char * QVImage<sShort,3>::getTypeQString()	const	{ return "QVImage<sShort,3>"; }
template <> const char * QVImage<sInt,1>::getTypeQString()	const	{ return "QVImage<sInt,1>"; }
template <> const char * QVImage<sInt,3>::getTypeQString()	const	{ return "QVImage<sInt,3>"; }
template <> const char * QVImage<sFloat,1>::getTypeQString()	const	{ return "QVImage<sFloat,1>"; }
template <> const char * QVImage<sFloat,3>::getTypeQString()	const	{ return "QVImage<sFloat,3>"; }

// Copy constructors
#define	CREATE_COPY_CONSTRUCTOR(TYPE, C)							\
template <> QVImage<TYPE, C>::QVImage(QVImage<TYPE, C> const &img):QVGenericImage(img)		\
	{											\
	imageBuffer = img.imageBuffer;								\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	step_div_type_size = getStep()/sizeof(TYPE);						\
	}

CREATE_COPY_CONSTRUCTOR(uChar, 1);
CREATE_COPY_CONSTRUCTOR(uChar, 3);
CREATE_COPY_CONSTRUCTOR(uShort, 1);
CREATE_COPY_CONSTRUCTOR(uShort, 3);
CREATE_COPY_CONSTRUCTOR(sShort, 1);
CREATE_COPY_CONSTRUCTOR(sShort, 3);
CREATE_COPY_CONSTRUCTOR(sInt, 1);
CREATE_COPY_CONSTRUCTOR(sInt, 3);
CREATE_COPY_CONSTRUCTOR(sFloat, 1);
CREATE_COPY_CONSTRUCTOR(sFloat, 3);

// Copy operator
#define	CREATE_COPY_OPERATOR(TYPE, C)							\
template <> QVImage<TYPE,C> & QVImage<TYPE, C>::operator=(const QVImage<TYPE, C> &img) 	\
	{										\
	imageBuffer = img.imageBuffer;							\
	setROI(img.getROI()); setAnchor(img.getAnchor());				\
	step_div_type_size = getStep()/sizeof(TYPE);					\
	return *this;									\
	}

CREATE_COPY_OPERATOR(uChar, 1);
CREATE_COPY_OPERATOR(uChar, 3);
CREATE_COPY_OPERATOR(uShort, 1);
CREATE_COPY_OPERATOR(uShort, 3);
CREATE_COPY_OPERATOR(sShort, 1);
CREATE_COPY_OPERATOR(sShort, 3);
CREATE_COPY_OPERATOR(sInt, 1);
CREATE_COPY_OPERATOR(sInt, 3);
CREATE_COPY_OPERATOR(sFloat, 1);
CREATE_COPY_OPERATOR(sFloat, 3);

//////////////////////////////////////////////////////////////////
// QVIPP FUNCTIONS
#include <qvip/qvipp/qvipp.h>

// Compose copy constructors
#define	CREATE_COMPOSE_COPY_CONSTRUCTOR(TYPE)															\
template <> QVImage<TYPE, 3>::QVImage(QVImage<TYPE,1> const &red, QVImage<TYPE,1> const &green, QVImage<TYPE,1> const &blue):QVGenericImage(red)		\
	{																			\
	imageBuffer = new QVImageBuffer<TYPE, 3>(red.getCols(), red.getRows());											\
	Copy(red, green, blue, *this);																\
	setROI(red.getROI()); setAnchor(red.getAnchor());													\
	step_div_type_size = getStep()/sizeof(TYPE);														\
	}

CREATE_COMPOSE_COPY_CONSTRUCTOR(uChar);
CREATE_COMPOSE_COPY_CONSTRUCTOR(uShort);
CREATE_COMPOSE_COPY_CONSTRUCTOR(sShort);
CREATE_COMPOSE_COPY_CONSTRUCTOR(sInt);
CREATE_COMPOSE_COPY_CONSTRUCTOR(sFloat);

// Conversion constructors
#define	CREATE_CONVERT_CONSTRUCTOR(TYPE1, TYPE2, C)						\
template <> QVImage<TYPE2, C>::QVImage(QVImage<TYPE1, C> const &img):QVGenericImage(img)	\
	{											\
	imageBuffer = new QVImageBuffer<TYPE2, C>(img.getCols(), img.getRows());		\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	Convert(img, *this);									\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	step_div_type_size = getStep()/sizeof(TYPE2);						\
	}

// (ORIG, DEST, CHAN)
CREATE_CONVERT_CONSTRUCTOR(uChar,	uShort, 1);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sShort, 1);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sInt, 1);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sFloat, 1);
CREATE_CONVERT_CONSTRUCTOR(uChar,	uShort, 3);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sShort, 3);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sInt, 3);
CREATE_CONVERT_CONSTRUCTOR(uChar,	sFloat, 3);
CREATE_CONVERT_CONSTRUCTOR(uShort,	uChar, 1);
CREATE_CONVERT_CONSTRUCTOR(uShort,	sInt, 1);
CREATE_CONVERT_CONSTRUCTOR(uShort,	sFloat, 1);
CREATE_CONVERT_CONSTRUCTOR(uShort,	uChar, 3);
CREATE_CONVERT_CONSTRUCTOR(uShort,	sInt, 3);
CREATE_CONVERT_CONSTRUCTOR(uShort,	sFloat, 3);
CREATE_CONVERT_CONSTRUCTOR(sShort,	uChar, 1);
CREATE_CONVERT_CONSTRUCTOR(sShort,	sInt, 1);
CREATE_CONVERT_CONSTRUCTOR(sShort,	sFloat, 1);
CREATE_CONVERT_CONSTRUCTOR(sShort,	uChar, 3);
CREATE_CONVERT_CONSTRUCTOR(sShort,	sInt, 3);
CREATE_CONVERT_CONSTRUCTOR(sShort,	sFloat, 3);
CREATE_CONVERT_CONSTRUCTOR(sInt,	uChar, 1);
CREATE_CONVERT_CONSTRUCTOR(sInt,	uChar, 3);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	uChar, 1);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	uShort, 1);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	sShort, 1);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	uChar, 3);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	uShort, 3);
CREATE_CONVERT_CONSTRUCTOR(sFloat,	sShort, 3);

// Nota: las siguientes funciones dan error al compilar, ya que no existen versiones de la función wrapper Convert,
// para convertir de una imagen entera, a una imagen de tipo uShort, sShort, o sFloat.
//	CREATE_CONVERT_CONSTRUCTOR(uShort, sInt, 1);
//	CREATE_CONVERT_CONSTRUCTOR(sShort, sInt, 1);
//	CREATE_CONVERT_CONSTRUCTOR(uShort, sInt, 3);
//	CREATE_CONVERT_CONSTRUCTOR(sShort, sInt, 3);
//	CREATE_CONVERT_CONSTRUCTOR(sFloat, sInt, 3);
//	CREATE_CONVERT_CONSTRUCTOR(sFloat, sInt, 3);
//
// Por ello, estas funciones deben ser definidas con funciones de conversión Ad-hoc, así como las funciones de conversión
// de valores uShort a valores sShort, y viceversa, que tampoco están implementadas por las IPP, ni las funciones para convertir
// imagenes de distinto tipo y canal.
//
// Ojo: Para implementar las funciones de conversión entre imagenes de tipo sShort y uShort, hay que tener en cuenta la saturación
// (que hacen las IPP para todas las funciones de conversión que implementa):
// - Si se quiere convertir un valor sShort menor que cero a un valor uShort, se deja como el valor cero (0).
// - Si se quiere convertir un valor uShort mayor que IPP_MAX_16S a sShort, se debe dejar como el valor IPP_MAX_16S.

#define	CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(TYPE1, TYPE2)						\
template <> QVImage<TYPE2, 1>::QVImage(QVImage<TYPE1, 1> const &img):QVGenericImage(img)		\
	{												\
	imageBuffer = new QVImageBuffer<TYPE2, 1>(img.getCols(), img.getRows());			\
	QVIMAGE_PTR_INIT_WRITE(TYPE2, this);								\
	QVIMAGE_INIT_READ(TYPE1, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 0);		\
 	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	step_div_type_size = getStep()/sizeof(TYPE2);							\
	}

CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(uShort, sShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(sShort, uShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(sInt, uShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(sInt, sShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(sInt, sFloat);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_1(sFloat, sInt);

#define	CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(TYPE1, TYPE2)						\
template <> QVImage<TYPE2, 3>::QVImage(QVImage<TYPE1, 3> const &img):QVGenericImage(img)		\
	{												\
	imageBuffer = new QVImageBuffer<TYPE2, 3>(img.getCols(), img.getRows());			\
	QVIMAGE_PTR_INIT_WRITE(TYPE2, this);								\
	QVIMAGE_INIT_READ(TYPE1, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			{										\
			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 0);		\
			QVIMAGE_PIXEL(this, col, row, 1) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 1);		\
			QVIMAGE_PIXEL(this, col, row, 2) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 2);		\
			}										\
 	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	step_div_type_size = getStep()/sizeof(TYPE2);							\
	}

CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(uShort, sShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(sShort, uShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(sInt, uShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(sInt, sShort);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(sInt, sFloat);
CREATE_CONVERT_CONSTRUCTOR_NO_IPP_3(sFloat, sInt);

#define	CREATE_CONVERT_CONSTRUCTOR_C3_C1(TYPE)							\
template <> QVImage<TYPE, 1>::QVImage(QVImage<TYPE, 3> const &img):QVGenericImage(img)		\
	{											\
	imageBuffer = new QVImageBuffer<TYPE, 1>(img.getCols(), img.getRows());			\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	RGBToGray(img, *this);									\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	}

CREATE_CONVERT_CONSTRUCTOR_C3_C1(uChar);
CREATE_CONVERT_CONSTRUCTOR_C3_C1(uShort);
CREATE_CONVERT_CONSTRUCTOR_C3_C1(sShort);
CREATE_CONVERT_CONSTRUCTOR_C3_C1(sFloat);

#define	CREATE_CONVERT_CONSTRUCTOR_NO_IPP_C3_C1(TYPE)							\
template <> QVImage<TYPE, 1>::QVImage(QVImage<TYPE, 3> const &img):QVGenericImage(img)			\
	{												\
	imageBuffer = new QVImageBuffer<TYPE, 1>(img.getCols(), img.getRows());				\
	QVIMAGE_PTR_INIT_WRITE(TYPE, this);								\
	QVIMAGE_INIT_READ(TYPE, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
 		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			{										\
 			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE)(0.299 * QVIMAGE_PIXEL(img, col, row, 0));	\
 			QVIMAGE_PIXEL(this, col, row, 0) += (TYPE)(0.587 * QVIMAGE_PIXEL(img, col, row, 1));	\
 			QVIMAGE_PIXEL(this, col, row, 0) += (TYPE)(0.114 * QVIMAGE_PIXEL(img, col, row, 2));	\
			}										\
  	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	step_div_type_size = getStep()/sizeof(TYPE);							\
	}

CREATE_CONVERT_CONSTRUCTOR_NO_IPP_C3_C1(sInt);

#define	CREATE_CONVERT_CONSTRUCTOR_C1_C3(TYPE)							\
template <> QVImage<TYPE, 3>::QVImage(QVImage<TYPE, 1> const &img):QVGenericImage(img)		\
	{											\
	imageBuffer = new QVImageBuffer<TYPE, 3>(img.getCols(), img.getRows());			\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	Copy(img, img, img, *this);								\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	step_div_type_size = getStep()/sizeof(TYPE);						\
	}

CREATE_CONVERT_CONSTRUCTOR_C1_C3(uChar);
CREATE_CONVERT_CONSTRUCTOR_C1_C3(uShort);
CREATE_CONVERT_CONSTRUCTOR_C1_C3(sShort);
CREATE_CONVERT_CONSTRUCTOR_C1_C3(sInt);
CREATE_CONVERT_CONSTRUCTOR_C1_C3(sFloat);

// Convert to and from QImage
inline int align(int v, int m) 
	{ return (v%m==0) ? v:(v-v%m+m); }

template <> QVImage<uChar, 3>::QVImage(const QImage &qimg):QVGenericImage()
	{
	imageBuffer = new QVImageBuffer<uChar, 3>(qimg.size().width(), qimg.size().height(), align(3*qimg.size().width(),4));
	setROI(0, 0, qimg.size().width(), qimg.size().height());
	setAnchor(getROI().x(), getROI().y());

	const int step = getStep();
	uChar *imagerawdata = getWriteData();
	for(uint row=0; row != getRows(); row++)
	       for(uint col=0; col != getCols(); col++)
			{
			imagerawdata[row*step + 3*col] = qRed(qimg.pixel(col,row));
			imagerawdata[row*step + 3*col + 1] = qGreen(qimg.pixel(col,row));
			imagerawdata[row*step + 3*col + 2] = qBlue(qimg.pixel(col,row));
			}
	}

template <> QVImage<uChar, 3>::operator QImage() const
	{
        QRect roi = getROI();
        const int x=roi.x(), y=roi.y(), w=roi.width(), h=roi.height(), step = getStep();
        QImage qImage(w, h, QImage::Format_RGB32);
	const uChar *imagerawdata = getReadData();
        for(int row = y; row != y+h; row++)
               for(int col = x; col != x+w; col++)
			{
                        uChar r = imagerawdata[(row-y)*step + 3*(col-x)],
                              g = imagerawdata[(row-y)*step + 3*(col-x) + 1],
                              b = imagerawdata[(row-y)*step + 3*(col-x) + 2];
                        qImage.setPixel(col-x, row-y, qRgb(r,g,b));
			}

	return qImage;
	}

// Convert operators for IplImage
#ifdef OPENCV
/*template <> QVImage<uChar, 3>::QVImage(const IplImage *iplImage):QVGenericImage()
	{
	imageBuffer = new QVImageBuffer<uChar, 3>(iplImage->width, iplImage->height, align(3*iplImage->width,4));
	setROI(0, 0, iplImage->width, iplImage->height);
	setAnchor(0, 0);

	const int lineSize = getCols() * 3;	// This depends on data size

	for (uInt y=0 ;y<getRows(); y++)
		memcpy(getWriteData() + getStep() * y, iplImage->imageData + iplImage->widthStep*y, lineSize);
	}*/

/*template <> QVImage<uChar, 3>::operator IplImage *() const
	{
	IplImage *imageDest = cvCreateImageHeader(cvSize(getCols(), getRows()), IPL_DEPTH_8U, 3);
	cvCreateImageData(imageDest);

	const int lineSize = getCols() * 3;	// This depends on data size

	for (uInt y=0 ;y<getRows(); y++)
		memcpy(imageDest->imageData + imageDest->widthStep*y, getReadData() + getStep() * y, lineSize);

	return imageDest;
	}*/

// IPL_DEPTH_8U
// IPL_DEPTH_8S
// IPL_DEPTH_16U
// IPL_DEPTH_16S
// IPL_DEPTH_32S
// IPL_DEPTH_32F
// IPL_DEPTH_64F

/// @todo Check IplImage is compatible with actual image
#define	CREATE_IPLIMAGE_CONSTRUCTOR(TYPE, C, DEPTH)									\
template <> QVImage<TYPE, C>::QVImage(const IplImage *iplImage):QVGenericImage()					\
	{														\
	if (iplImage->nChannels != C)											\
		qWarning("Conversion from incompatible OpenCV image: incompatible channel number");			\
	if (iplImage->depth != DEPTH)											\
		qWarning("Conversion from incompatible OpenCV image: incompatible depth");				\
															\
	imageBuffer = new QVImageBuffer<TYPE, C>(iplImage->width, iplImage->height, align(C*iplImage->width,4));	\
	setROI(0, 0, iplImage->width, iplImage->height);								\
	setAnchor(0, 0);												\
															\
	const int lineSize = getCols() * C;	/* This depends on data size */						\
															\
	for (uInt y=0 ;y<getRows(); y++)										\
		memcpy(getWriteData() + getStep() * y, iplImage->imageData + iplImage->widthStep*y, lineSize);		\
	}

CREATE_IPLIMAGE_CONSTRUCTOR(uChar, 1, IPL_DEPTH_8U);
CREATE_IPLIMAGE_CONSTRUCTOR(uChar, 3, IPL_DEPTH_8U);
CREATE_IPLIMAGE_CONSTRUCTOR(sFloat, 1, IPL_DEPTH_32F);
CREATE_IPLIMAGE_CONSTRUCTOR(sFloat, 3, IPL_DEPTH_32F);

#define	CREATE_IPLIMAGE_CONVERSION(TYPE, C, DEPTH)									\
template <> QVImage<TYPE, C>::operator IplImage *() const								\
	{														\
	IplImage *imageDest = cvCreateImageHeader(cvSize(getCols(), getRows()), DEPTH, C);				\
	cvCreateImageData(imageDest);											\
															\
	const int lineSize = getCols() * C;	/* This depends on data size */						\
															\
	for (uInt y=0 ;y<getRows(); y++)										\
		memcpy(imageDest->imageData + imageDest->widthStep*y, getReadData() + getStep() * y, lineSize);		\
															\
	return imageDest;												\
	}

CREATE_IPLIMAGE_CONVERSION(uChar, 1, IPL_DEPTH_8U);
CREATE_IPLIMAGE_CONVERSION(uChar, 3, IPL_DEPTH_8U);
CREATE_IPLIMAGE_CONVERSION(sFloat, 1, IPL_DEPTH_32F);
CREATE_IPLIMAGE_CONVERSION(sFloat, 3, IPL_DEPTH_32F);

#endif

// Operators
#define CREATE_OPERATOR(NAME, OPERATOR, TYPE, C)						\
template <> QVImage<TYPE, C> QVImage<TYPE,C>::OPERATOR(const QVImage<TYPE, C> &img) const	\
	{											\
	QVImage<TYPE, C> result = *this;							\
	NAME(*this, img, result);								\
	return result;										\
	}

CREATE_OPERATOR(Add, operator+, uChar, 1);
CREATE_OPERATOR(Mul, operator*, uChar, 1);
CREATE_OPERATOR(Sub, operator-, uChar, 1);
CREATE_OPERATOR(Div, operator/, uChar, 1);
CREATE_OPERATOR(Add, operator+, uChar, 3);
CREATE_OPERATOR(Mul, operator*, uChar, 3);
CREATE_OPERATOR(Sub, operator-, uChar, 3);
CREATE_OPERATOR(Div, operator/, uChar, 3);

CREATE_OPERATOR(Add, operator+, uShort, 1);
CREATE_OPERATOR(Mul, operator*, uShort, 1);
CREATE_OPERATOR(Sub, operator-, uShort, 1);
CREATE_OPERATOR(Div, operator/, uShort, 1);
CREATE_OPERATOR(Add, operator+, uShort, 3);
CREATE_OPERATOR(Mul, operator*, uShort, 3);
CREATE_OPERATOR(Sub, operator-, uShort, 3);
CREATE_OPERATOR(Div, operator/, uShort, 3);

CREATE_OPERATOR(Add, operator+, sShort, 1);
CREATE_OPERATOR(Mul, operator*, sShort, 1);
CREATE_OPERATOR(Sub, operator-, sShort, 1);
CREATE_OPERATOR(Div, operator/, sShort, 1);
CREATE_OPERATOR(Add, operator+, sShort, 3);
CREATE_OPERATOR(Mul, operator*, sShort, 3);
CREATE_OPERATOR(Sub, operator-, sShort, 3);
CREATE_OPERATOR(Div, operator/, sShort, 3);

#define CREATE_OPERATOR_INT_C1(OPERATOR, OPERATION, TYPE)											\
template <> QVImage<TYPE, 1> QVImage<TYPE, 1>::OPERATOR(const QVImage<TYPE, 1> &img) const							\
	{																	\
	QVImage<TYPE, 1> result = *this;													\
	QVIMAGE_PTR_INIT_READ(TYPE, this);													\
	QVIMAGE_INIT_READ(TYPE, img);														\
	QVIMAGE_INIT_WRITE(TYPE, result);													\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)								\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)							\
			QVIMAGE_PIXEL(result, col, row, 0) = QVIMAGE_PIXEL(img, col, row, 0) OPERATION QVIMAGE_PIXEL(this, col, row, 0);	\
	return result;																\
	}

CREATE_OPERATOR_INT_C1(operator+, +, sInt);
CREATE_OPERATOR_INT_C1(operator*, *, sInt);
CREATE_OPERATOR_INT_C1(operator-, -, sInt);
CREATE_OPERATOR_INT_C1(operator/, /, sInt);

#define CREATE_OPERATOR_INT_C3(OPERATOR, OPERATION, TYPE)											\
template <> QVImage<TYPE, 3> QVImage<TYPE, 3>::OPERATOR(const QVImage<TYPE, 3> &img) const							\
	{																	\
	QVImage<TYPE, 3> result = *this;													\
	QVIMAGE_PTR_INIT_READ(TYPE, this);													\
	QVIMAGE_INIT_READ(TYPE, img);														\
	QVIMAGE_INIT_WRITE(TYPE, result);													\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)								\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)							\
			{															\
			QVIMAGE_PIXEL(result, col, row, 0) = QVIMAGE_PIXEL(img, col, row, 0) OPERATION QVIMAGE_PIXEL(this, col, row, 0);	\
			QVIMAGE_PIXEL(result, col, row, 1) = QVIMAGE_PIXEL(img, col, row, 1) OPERATION QVIMAGE_PIXEL(this, col, row, 1);	\
			QVIMAGE_PIXEL(result, col, row, 2) = QVIMAGE_PIXEL(img, col, row, 2) OPERATION QVIMAGE_PIXEL(this, col, row, 2);	\
			}															\
	return result;																\
	}

CREATE_OPERATOR_INT_C3(operator+, +, sInt);
CREATE_OPERATOR_INT_C3(operator*, *, sInt);
CREATE_OPERATOR_INT_C3(operator-, -, sInt);
CREATE_OPERATOR_INT_C3(operator/, /, sInt);


#define CREATE_CONST_OPERATOR(NAME, OPERATOR, TYPE, C)				\
template <> QVImage<TYPE, C> QVImage<TYPE,C>::OPERATOR(const TYPE value) const	\
	{									\
	QVImage<TYPE, C> result = *this;					\
	NAME(*this, value, result);						\
	return result;								\
	}

CREATE_CONST_OPERATOR(AddC, operator+, uChar, 1);
CREATE_CONST_OPERATOR(MulC, operator*, uChar, 1);
CREATE_CONST_OPERATOR(SubC, operator-, uChar, 1);
CREATE_CONST_OPERATOR(DivC, operator/, uChar, 1);
CREATE_CONST_OPERATOR(AddC, operator+, uShort, 1);
CREATE_CONST_OPERATOR(MulC, operator*, uShort, 1);
CREATE_CONST_OPERATOR(SubC, operator-, uShort, 1);
CREATE_CONST_OPERATOR(DivC, operator/, uShort, 1);
CREATE_CONST_OPERATOR(AddC, operator+, sShort, 1);
CREATE_CONST_OPERATOR(MulC, operator*, sShort, 1);
CREATE_CONST_OPERATOR(SubC, operator-, sShort, 1);
CREATE_CONST_OPERATOR(DivC, operator/, sShort, 1);
CREATE_CONST_OPERATOR(AddC, operator+, sFloat, 1);
CREATE_CONST_OPERATOR(MulC, operator*, sFloat, 1);
CREATE_CONST_OPERATOR(SubC, operator-, sFloat, 1);
CREATE_CONST_OPERATOR(DivC, operator/, sFloat, 1);
CREATE_CONST_OPERATOR(LShiftC, operator<<, uChar, 1);
CREATE_CONST_OPERATOR(RShiftC, operator>>, uChar, 1);
CREATE_CONST_OPERATOR(LShiftC, operator<<, uShort, 1);
CREATE_CONST_OPERATOR(RShiftC, operator>>, uShort, 1);
CREATE_CONST_OPERATOR(LShiftC, operator<<, sInt, 1);
CREATE_CONST_OPERATOR(RShiftC, operator>>, sInt, 1);
CREATE_CONST_OPERATOR(AndC, operator&, uChar, 1);
CREATE_CONST_OPERATOR(OrC, operator|, uChar, 1);
CREATE_CONST_OPERATOR(XorC, operator^, uChar, 1);
CREATE_CONST_OPERATOR(AndC, operator&, uShort, 1);
CREATE_CONST_OPERATOR(OrC, operator|, uShort, 1);
CREATE_CONST_OPERATOR(XorC, operator^, uShort, 1);
CREATE_CONST_OPERATOR(AndC, operator&, sInt, 1);
CREATE_CONST_OPERATOR(OrC, operator|, sInt, 1);
CREATE_CONST_OPERATOR(XorC, operator^, sInt, 1);

/////////////////////////

#define CREATE_CONST_OPERATOR_C3(NAME, OPERATOR, TYPE, C)			\
template <> QVImage<TYPE, C> QVImage<TYPE,C>::OPERATOR(const TYPE value) const	\
	{									\
	const TYPE values[3] = { value, value, value };				\
	QVImage<TYPE, C> result = *this;					\
	NAME(*this, values, result);						\
	return result;								\
	}

CREATE_CONST_OPERATOR_C3(AddC, operator+, uChar, 3);
CREATE_CONST_OPERATOR_C3(MulC, operator*, uChar, 3);
CREATE_CONST_OPERATOR_C3(SubC, operator-, uChar, 3);
CREATE_CONST_OPERATOR_C3(DivC, operator/, uChar, 3);
CREATE_CONST_OPERATOR_C3(AddC, operator+, sShort, 3);
CREATE_CONST_OPERATOR_C3(MulC, operator*, sShort, 3);
CREATE_CONST_OPERATOR_C3(SubC, operator-, sShort, 3);
CREATE_CONST_OPERATOR_C3(DivC, operator/, sShort, 3);
CREATE_CONST_OPERATOR_C3(AddC, operator+, uShort, 3);
CREATE_CONST_OPERATOR_C3(MulC, operator*, uShort, 3);
CREATE_CONST_OPERATOR_C3(SubC, operator-, uShort, 3);
CREATE_CONST_OPERATOR_C3(DivC, operator/, uShort, 3);
CREATE_CONST_OPERATOR_C3(AddC, operator+, sFloat, 3);
CREATE_CONST_OPERATOR_C3(MulC, operator*, sFloat, 3);
CREATE_CONST_OPERATOR_C3(SubC, operator-, sFloat, 3);
CREATE_CONST_OPERATOR_C3(DivC, operator/, sFloat, 3);
CREATE_CONST_OPERATOR_C3(AndC, operator&, uChar, 3);
CREATE_CONST_OPERATOR_C3(OrC, operator|, uChar, 3);
CREATE_CONST_OPERATOR_C3(XorC, operator^, uChar, 3);
CREATE_CONST_OPERATOR_C3(AndC, operator&, uShort, 3);
CREATE_CONST_OPERATOR_C3(OrC, operator|, uShort, 3);
CREATE_CONST_OPERATOR_C3(XorC, operator^, uShort, 3);
CREATE_CONST_OPERATOR_C3(AndC, operator&, sInt, 3);
CREATE_CONST_OPERATOR_C3(OrC, operator|, sInt, 3);
CREATE_CONST_OPERATOR_C3(XorC, operator^, sInt, 3);

#define CREATE_CONST_OPERATOR_C4(NAME, OPERATOR, TYPE, C)			\
template <> QVImage<TYPE, C> QVImage<TYPE,C>::OPERATOR(const TYPE value) const	\
	{									\
	const uInt values[3] = { value, value, value };				\
	QVImage<TYPE, C> result = *this;					\
	NAME(*this, values, result);						\
	return result;								\
	}

CREATE_CONST_OPERATOR_C4(LShiftC, operator<<, uChar, 3);
CREATE_CONST_OPERATOR_C4(RShiftC, operator>>, uChar, 3);
CREATE_CONST_OPERATOR_C4(LShiftC, operator<<, uShort, 3);
CREATE_CONST_OPERATOR_C4(RShiftC, operator>>, uShort, 3);
CREATE_CONST_OPERATOR_C4(LShiftC, operator<<, sInt, 3);
CREATE_CONST_OPERATOR_C4(RShiftC, operator>>, sInt, 3);

#define CREATE_CONST_OPERATOR_INT_C1(OPERATOR, OPERATION, TYPE)							\
template <> QVImage<TYPE, 1> QVImage<TYPE, 1>::OPERATOR(const TYPE value) const					\
	{													\
	QVImage<TYPE, 1> result = *this;									\
	QVIMAGE_PTR_INIT_READ(TYPE, this);									\
	QVIMAGE_INIT_WRITE(TYPE, result);									\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)				\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)			\
			QVIMAGE_PIXEL(result, col, row, 0) = QVIMAGE_PIXEL(this, col, row, 0) OPERATION value;	\
	return result;												\
	}

CREATE_CONST_OPERATOR_INT_C1(operator+, +, sInt);
CREATE_CONST_OPERATOR_INT_C1(operator*, *, sInt);
CREATE_CONST_OPERATOR_INT_C1(operator-, -, sInt);
CREATE_CONST_OPERATOR_INT_C1(operator/, /, sInt);

#define CREATE_CONST_OPERATOR_INT_C3(OPERATOR, OPERATION, TYPE)							\
template <> QVImage<TYPE, 3> QVImage<TYPE, 3>::OPERATOR(const TYPE value) const					\
	{													\
	QVImage<TYPE, 3> result = *this;									\
	QVIMAGE_PTR_INIT_READ(TYPE, this);									\
	QVIMAGE_INIT_WRITE(TYPE, result);									\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)				\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)			\
			{											\
			QVIMAGE_PIXEL(result, col, row, 0) = QVIMAGE_PIXEL(this, col, row, 0) OPERATION value;	\
			QVIMAGE_PIXEL(result, col, row, 1) = QVIMAGE_PIXEL(this, col, row, 1) OPERATION value;	\
			QVIMAGE_PIXEL(result, col, row, 2) = QVIMAGE_PIXEL(this, col, row, 2) OPERATION value;	\
			}											\
	return result;												\
	}

CREATE_CONST_OPERATOR_INT_C3(operator+, +, sInt);
CREATE_CONST_OPERATOR_INT_C3(operator*, *, sInt);
CREATE_CONST_OPERATOR_INT_C3(operator-, -, sInt);
CREATE_CONST_OPERATOR_INT_C3(operator/, /, sInt);

#define CREATE_NOT_OPERATOR(NAME, OPERATOR, TYPE, C)		\
template <> QVImage<TYPE, C> QVImage<TYPE,C>::OPERATOR() const	\
	{							\
	QVImage<TYPE, C> result = *this;			\
	NAME(*this, result);					\
	return result;						\
	}

CREATE_NOT_OPERATOR(Not, operator!, uChar, 1);
CREATE_NOT_OPERATOR(Not, operator!, uChar, 3);

#define CREATE_COMPARE_OPERATOR(NAME, OPERATOR, CMP, TYPE, C)				\
template <> QVImage<uChar> QVImage<TYPE,C>::OPERATOR(const QVImage<TYPE, C> &img) const	\
	{										\
	QVImage<uChar> result(getCols(), getRows());					\
	NAME(*this, img, result, CMP);							\
	return result;									\
	}

// ippcmpLess
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, uChar, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, uShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, sShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, sFloat, 1);

CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, uChar, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, uShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, sShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<, ippCmpLess, sFloat, 3);

// ippCmpGreater
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, uChar, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, uShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, sShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, sFloat, 1);

CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, uChar, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, uShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, sShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>, ippCmpGreater, sFloat, 3);

// ippcmpLessEq
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, uChar, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, uShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, sShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, sFloat, 1);

CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, uChar, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, uShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, sShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator<=, ippCmpLessEq, sFloat, 3);

// ippCmpGreaterEq
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, uChar, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, uShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, sShort, 1);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, sFloat, 1);

CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, uChar, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, uShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, sShort, 3);
CREATE_COMPARE_OPERATOR(Compare, operator>=, ippCmpGreaterEq, sFloat, 3);

#define CREATE_GENERIC_COMPARE_OPERATOR_C1(OPERATOR, OPERATION, TYPE1, TYPE2)				\
template <> QVImage<uChar> QVImage<TYPE1, 1>::OPERATOR(const QVImage<TYPE2, 1> &img) const		\
	{												\
	QVImage<uChar> result(getCols(), getRows());							\
	result.setROI(getROI()); result.setAnchor(getAnchor());						\
	QVIMAGE_PTR_INIT_READ(TYPE1, this);								\
	QVIMAGE_INIT_READ(TYPE2, img);									\
	QVIMAGE_INIT_WRITE(uChar, result);								\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)			\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)		\
			if (QVIMAGE_PIXEL(this, col, row, 0) OPERATION QVIMAGE_PIXEL(img, col, row, 0))	\
				QVIMAGE_PIXEL(result, col, row, 0) = IPP_MAX_8U;			\
			else	QVIMAGE_PIXEL(result, col, row, 0) = 0;					\
	return result;											\
	}

CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uChar, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, uShort, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sShort, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sInt, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<, <, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>, >, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator<=, <=, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C1(operator>=, >=, sFloat, sInt);


#define CREATE_GENERIC_COMPARE_OPERATOR_C3(OPERATOR, OPERATION, TYPE1, TYPE2)					\
template <> QVImage<uChar> QVImage<TYPE1, 3>::OPERATOR(const QVImage<TYPE2, 3> &img) const			\
	{													\
	QVImage<uChar> result(getCols(), getRows());								\
	result.setROI(getROI()); result.setAnchor(getAnchor());							\
	QVIMAGE_PTR_INIT_READ(TYPE1, this);									\
	QVIMAGE_INIT_READ(TYPE2, img);										\
	QVIMAGE_INIT_WRITE(uChar, result);									\
	for (int col = result.getROI().left(); col < result.getROI().right(); col++)				\
		for (int row = result.getROI().top(); row < result.getROI().bottom(); row++)			\
			{											\
			if( (QVIMAGE_PIXEL(this, col, row, 0) OPERATION QVIMAGE_PIXEL(img, col, row, 0)) &&	\
			    (QVIMAGE_PIXEL(this, col, row, 1) OPERATION QVIMAGE_PIXEL(img, col, row, 1)) &&	\
			    (QVIMAGE_PIXEL(this, col, row, 2) OPERATION QVIMAGE_PIXEL(img, col, row, 2))    )	\
				QVIMAGE_PIXEL(result, col, row, 0) = IPP_MAX_8U;				\
			else	QVIMAGE_PIXEL(result, col, row, 0) = 0;						\
			}											\
	return result;												\
	}

CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uChar, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uChar, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uChar, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uChar, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uChar, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uShort, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, uShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, uShort, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sShort, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sShort, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sShort, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sShort, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sShort, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sInt, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sInt, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sInt, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sInt, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sInt, sFloat);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sInt, sFloat);

CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sFloat, uChar);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sFloat, uShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sFloat, sShort);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<, <, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>, >, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator<=, <=, sFloat, sInt);
CREATE_GENERIC_COMPARE_OPERATOR_C3(operator>=, >=, sFloat, sInt);


// Set FUNCTION
#define	CREATE_SET_FUNCTION_C1(TYPE)				\
template <> void QVImage<TYPE>::set(TYPE c1, TYPE, TYPE)	\
	{ Set(c1, *this); }

CREATE_SET_FUNCTION_C1(uChar);
CREATE_SET_FUNCTION_C1(uShort);
CREATE_SET_FUNCTION_C1(sShort);
CREATE_SET_FUNCTION_C1(sInt);
CREATE_SET_FUNCTION_C1(sFloat);

#define	CREATE_SET_FUNCTION_C3(TYPE)					\
template <> void QVImage<TYPE,3>::set(TYPE c1, TYPE c2, TYPE c3)	\
	{								\
	const TYPE values[3] = { c1, c2, c3 };				\
	Set(values, *this);						\
	}

CREATE_SET_FUNCTION_C3(uChar);
CREATE_SET_FUNCTION_C3(uShort);
CREATE_SET_FUNCTION_C3(sShort);
CREATE_SET_FUNCTION_C3(sInt);
CREATE_SET_FUNCTION_C3(sFloat);

//////////////
// operator =
#define	CREATE_CONVERT_OPERATOR(TYPE1, TYPE2, C)						\
template <> QVImage<TYPE2, C> & QVImage<TYPE2, C>::operator=(const QVImage<TYPE1, C> &img)	\
	{											\
	imageBuffer = new QVImageBuffer<TYPE2, C>(img.getCols(), img.getRows());		\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	Convert(img, *this);									\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	return *this;										\
	}

CREATE_CONVERT_OPERATOR(uChar, uShort, 1);
CREATE_CONVERT_OPERATOR(uChar, sShort, 1);
CREATE_CONVERT_OPERATOR(uChar, sInt, 1);
CREATE_CONVERT_OPERATOR(uChar, sFloat, 1);
CREATE_CONVERT_OPERATOR(uChar, uShort, 3);
CREATE_CONVERT_OPERATOR(uChar, sShort, 3);
CREATE_CONVERT_OPERATOR(uChar, sInt, 3);
CREATE_CONVERT_OPERATOR(uChar, sFloat, 3);
CREATE_CONVERT_OPERATOR(uShort, uChar, 1);
CREATE_CONVERT_OPERATOR(uShort, sInt, 1);
CREATE_CONVERT_OPERATOR(uShort, sFloat, 1);
CREATE_CONVERT_OPERATOR(uShort, uChar, 3);
CREATE_CONVERT_OPERATOR(uShort, sInt, 3);
CREATE_CONVERT_OPERATOR(uShort, sFloat, 3);
CREATE_CONVERT_OPERATOR(sShort, uChar, 1);
CREATE_CONVERT_OPERATOR(sShort, sInt, 1);
CREATE_CONVERT_OPERATOR(sShort, sFloat, 1);
CREATE_CONVERT_OPERATOR(sShort, uChar, 3);
CREATE_CONVERT_OPERATOR(sShort, sInt, 3);
CREATE_CONVERT_OPERATOR(sShort, sFloat, 3);
CREATE_CONVERT_OPERATOR(sInt, uChar, 1);
CREATE_CONVERT_OPERATOR(sInt, uChar, 3);
CREATE_CONVERT_OPERATOR(sFloat, uChar, 1);
CREATE_CONVERT_OPERATOR(sFloat, uShort, 1);
CREATE_CONVERT_OPERATOR(sFloat, sShort, 1);
CREATE_CONVERT_OPERATOR(sFloat, uChar, 3);
CREATE_CONVERT_OPERATOR(sFloat, uShort, 3);
CREATE_CONVERT_OPERATOR(sFloat, sShort, 3);

// Nota: las siguientes funciones dan error al compilar, ya que no existen versiones de la función wrapper Convert,
// para convertir de una imagen entera, a una imagen de tipo uShort, sShort, o sFloat.
//	CREATE_CONVERT_OPERATOR(uShort, sInt, 1);
//	CREATE_CONVERT_OPERATOR(sShort, sInt, 1);
//	CREATE_CONVERT_OPERATOR(uShort, sInt, 3);
//	CREATE_CONVERT_OPERATOR(sShort, sInt, 3);
//	CREATE_CONVERT_OPERATOR(sFloat, sInt, 3);
//	CREATE_CONVERT_OPERATOR(sFloat, sInt, 3);
//
// Por ello, estas funciones deben ser definidas con funciones de conversión Ad-hoc, así como las funciones de conversión
// de valores uShort a valores sShort, y viceversa, que tampoco están implementadas por las IPP, ni las funciones para convertir
// imagenes de distinto tipo y canal.
//
// Ojo: Para implementar las funciones de conversión entre imagenes de tipo sShort y uShort, hay que tener en cuenta la saturación
// (que hacen las IPP para todas las funciones de conversión que implementa):
// - Si se quiere convertir un valor sShort menor que cero a un valor uShort, se deja como el valor cero (0).
// - Si se quiere convertir un valor uShort mayor que IPP_MAX_16S a sShort, se debe dejar como el valor IPP_MAX_16S.


#define	CREATE_CONVERT_OPERATOR_NO_IPP_1(TYPE1, TYPE2)							\
template <> QVImage<TYPE2, 1> & QVImage<TYPE2, 1>::operator=(const QVImage<TYPE1, 1> &img)		\
	{												\
	imageBuffer = new QVImageBuffer<TYPE2, 1>(img.getCols(), img.getRows());			\
	QVIMAGE_PTR_INIT_WRITE(TYPE2, this);								\
	QVIMAGE_INIT_READ(TYPE1, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 0);		\
 	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	return *this;											\
	}

CREATE_CONVERT_OPERATOR_NO_IPP_1(uShort, sShort);
CREATE_CONVERT_OPERATOR_NO_IPP_1(sShort, uShort);
CREATE_CONVERT_OPERATOR_NO_IPP_1(sInt, uShort);
CREATE_CONVERT_OPERATOR_NO_IPP_1(sInt, sShort);
CREATE_CONVERT_OPERATOR_NO_IPP_1(sInt, sFloat);
CREATE_CONVERT_OPERATOR_NO_IPP_1(sFloat, sInt);

#define	CREATE_CONVERT_OPERATOR_NO_IPP_3(TYPE1, TYPE2)							\
template <> QVImage<TYPE2, 3> & QVImage<TYPE2, 3>::operator=(const QVImage<TYPE1, 3> &img)		\
	{												\
	imageBuffer = new QVImageBuffer<TYPE2, 3>(img.getCols(), img.getRows());			\
	QVIMAGE_PTR_INIT_WRITE(TYPE2, this);								\
	QVIMAGE_INIT_READ(TYPE1, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			{										\
			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 0);		\
			QVIMAGE_PIXEL(this, col, row, 1) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 1);		\
			QVIMAGE_PIXEL(this, col, row, 2) = (TYPE2)QVIMAGE_PIXEL(img, col, row, 2);		\
			}										\
 	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	return *this;											\
	}

CREATE_CONVERT_OPERATOR_NO_IPP_3(uShort, sShort);
CREATE_CONVERT_OPERATOR_NO_IPP_3(sShort, uShort);
CREATE_CONVERT_OPERATOR_NO_IPP_3(sInt, uShort);
CREATE_CONVERT_OPERATOR_NO_IPP_3(sInt, sShort);
CREATE_CONVERT_OPERATOR_NO_IPP_3(sInt, sFloat);
CREATE_CONVERT_OPERATOR_NO_IPP_3(sFloat, sInt);

#define	CREATE_CONVERT_OPERATOR_C3_C1(TYPE)							\
template <> QVImage<TYPE, 1> & QVImage<TYPE, 1>::operator=(const QVImage<TYPE, 3> &img)		\
	{											\
	imageBuffer = new QVImageBuffer<TYPE, 1>(img.getCols(), img.getRows());			\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	RGBToGray(img, *this);									\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	return *this;										\
	}

CREATE_CONVERT_OPERATOR_C3_C1(uChar);
CREATE_CONVERT_OPERATOR_C3_C1(uShort);
CREATE_CONVERT_OPERATOR_C3_C1(sShort);
CREATE_CONVERT_OPERATOR_C3_C1(sFloat);

#define	CREATE_CONVERT_OPERATOR_NO_IPP_C3_C1(TYPE)							\
template <> QVImage<TYPE, 1> & QVImage<TYPE, 1>::operator=(const QVImage<TYPE, 3> &img)			\
	{												\
	imageBuffer = new QVImageBuffer<TYPE, 1>(img.getCols(), img.getRows());				\
	QVIMAGE_PTR_INIT_WRITE(TYPE, this);								\
	QVIMAGE_INIT_READ(TYPE, img);									\
	for (int col = img.getROI().left(); col < img.getROI().right(); col++)				\
 		for (int row = img.getROI().top(); row < img.getROI().bottom(); row++)			\
			{										\
 			QVIMAGE_PIXEL(this, col, row, 0) = (TYPE)(0.299 * QVIMAGE_PIXEL(img, col, row, 0));	\
 			QVIMAGE_PIXEL(this, col, row, 0) += (TYPE)(0.587 * QVIMAGE_PIXEL(img, col, row, 1));	\
 			QVIMAGE_PIXEL(this, col, row, 0) += (TYPE)(0.114 * QVIMAGE_PIXEL(img, col, row, 2));	\
			}										\
  	setROI(img.getROI()); setAnchor(img.getAnchor());						\
	return *this;											\
	}

CREATE_CONVERT_OPERATOR_NO_IPP_C3_C1(sInt);

#define	CREATE_CONVERT_OPERATOR_C1_C3(TYPE)							\
template <> QVImage<TYPE, 3> & QVImage<TYPE, 3>::operator=(const QVImage<TYPE, 1> &img)		\
	{											\
	imageBuffer = new QVImageBuffer<TYPE, 3>(img.getCols(), img.getRows());			\
	setAnchor(img.getROI().x(),img.getROI().y());						\
	Copy(img, img, img, *this);								\
	setROI(img.getROI()); setAnchor(img.getAnchor());					\
	return *this;										\
	}

CREATE_CONVERT_OPERATOR_C1_C3(uChar);
CREATE_CONVERT_OPERATOR_C1_C3(uShort);
CREATE_CONVERT_OPERATOR_C1_C3(sShort);
CREATE_CONVERT_OPERATOR_C1_C3(sInt);
CREATE_CONVERT_OPERATOR_C1_C3(sFloat);


//Convert constructors with double conversion (after operator= declaration)
#define	CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(TYPE1, TYPE2)					\
template <> QVImage<TYPE2, 1>::QVImage(QVImage<TYPE1, 3> const &img):QVGenericImage(img)	\
	{											\
	*this = QVImage<TYPE2, 3>(img);								\
	}

CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uChar, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uChar, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uChar, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uChar, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uShort, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uShort, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uShort, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(uShort, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sShort, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sShort, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sShort, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sShort, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sInt, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sInt, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sInt, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sInt, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sFloat, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sFloat, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sFloat, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C3_C1(sFloat, sInt);

#define	CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(TYPE1, TYPE2)					\
template <> QVImage<TYPE2, 3>::QVImage(QVImage<TYPE1, 1> const &img):QVGenericImage(img)	\
	{											\
	*this = QVImage<TYPE2, 1>(img);								\
	}

CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uChar, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uChar, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uChar, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uChar, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uShort, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uShort, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uShort, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(uShort, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sShort, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sShort, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sShort, sInt);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sShort, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sInt, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sInt, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sInt, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sInt, sFloat);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sFloat, uChar);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sFloat, uShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sFloat, sShort);
CREATE_CONVERT_CONSTRUCTOR_DOUBLE_C1_C3(sFloat, sInt);


#define	CREATE_ACCESS_CONSTRUCTOR(OPERATOR, TYPE, C)						\
template <> QVImage<TYPE, 1> QVImage<TYPE, C>::OPERATOR(const uInt channel) const		\
	{											\
	QVImage<TYPE, 1> result(getCols(), getRows());						\
	Copy(*this, channel, result);								\
	result.setROI(getROI()); result.setAnchor(getAnchor());					\
	return result;										\
	}

CREATE_ACCESS_CONSTRUCTOR(operator(), uChar, 1);
CREATE_ACCESS_CONSTRUCTOR(operator(), uChar, 3);
CREATE_ACCESS_CONSTRUCTOR(operator(), uShort, 1);
CREATE_ACCESS_CONSTRUCTOR(operator(), uShort, 3);
CREATE_ACCESS_CONSTRUCTOR(operator(), sShort, 1);
CREATE_ACCESS_CONSTRUCTOR(operator(), sShort, 3);
CREATE_ACCESS_CONSTRUCTOR(operator(), sInt, 1);
CREATE_ACCESS_CONSTRUCTOR(operator(), sInt, 3);
CREATE_ACCESS_CONSTRUCTOR(operator(), sFloat, 1);
CREATE_ACCESS_CONSTRUCTOR(operator(), sFloat, 3);




